"""API clients for Agent Office SDK."""

from .base import BaseAPI
from .documents import DocumentsAPI
from .edits import EditsAPI
from .markdown import MarkdownAPI

__all__ = ["BaseAPI", "DocumentsAPI", "EditsAPI", "MarkdownAPI"]




"""Data models for Agent Office SDK."""

from .document import (
    DocumentResponse,
    DocumentInfo,
    DocumentExistsResponse,
    ListDocumentsResponse,
    DownloadDocumentResponse,
    DocumentFromUrlRequest,
    ImageAnnotation,
)
from .edit import EditRequest, EditResponse
from .markdown import MarkdownReadRequest, MarkdownReadResponse

__all__ = [
    "DocumentResponse",
    "DocumentInfo",
    "DocumentExistsResponse",
    "ListDocumentsResponse",
    "DownloadDocumentResponse",
    "DocumentFromUrlRequest",
    "ImageAnnotation",
    "EditRequest",
    "EditResponse",
    "MarkdownReadRequest",
    "MarkdownReadResponse",
]




"""Main Agent Office client."""

from typing import BinaryIO, Optional, Union
from pathlib import Path
from uuid import uuid4

from .apis import DocumentsAPI, EditsAPI, MarkdownAPI
from .models import (
    DocumentResponse,
    DocumentExistsResponse,
    ListDocumentsResponse,
    DownloadDocumentResponse,
    DocumentFromUrlRequest,
    EditRequest,
    EditResponse,
    MarkdownReadRequest,
    MarkdownReadResponse,
)


class AgentOffice:
    """Main client for Agent Office API.

    Example:
        >>> from agent_office import AgentOffice
        >>> client = AgentOffice(api_key="sk_ao_your_api_key")
        >>>
        >>> # Upload a document
        >>> doc = client.documents.create("document.docx", return_markdown=True)
        >>> print(f"Uploaded: {doc.doc_id}")
        >>>
        >>> # Edit the document
        >>> edit = client.edit(
        ...     doc_id=doc.doc_id,
        ...     edit_uid=str(uuid4()),
        ...     edit_instructions="Change the title to 'My New Title'"
        ... )
        >>> print(f"Edit applied: {edit.edit_applied}")
    """

    def __init__(
        self,
        api_key: str,
        base_url: str = "https://api.agentoffice.dev",
        timeout: int = 60,
    ):
        """Initialize Agent Office client.

        Args:
            api_key: Your API key (get one at https://agentoffice.dev)
            base_url: Base URL for the API (default: https://api.agentoffice.dev)
            timeout: Request timeout in seconds (default: 60)
        """
        self.api_key = api_key
        self.base_url = base_url
        self.timeout = timeout

        # Initialize API clients
        self._documents_api = DocumentsAPI(base_url, api_key, timeout)
        self._edits_api = EditsAPI(base_url, api_key, timeout)
        self._markdown_api = MarkdownAPI(base_url, api_key, timeout)

    @property
    def documents(self) -> "DocumentsNamespace":
        """Access document operations."""
        return DocumentsNamespace(self._documents_api)

    def edit(
        self,
        doc_id: str,
        edit_uid: str,
        edit_instructions: str,
        lookup_text: Optional[str] = None,
        save_chunks_for_review: bool = False,
        use_large_model: bool = False,
    ) -> EditResponse:
        """Edit a document.

        Args:
            doc_id: Document ID
            edit_uid: Unique identifier for this edit (for idempotency)
            edit_instructions: Natural language edit instructions
            lookup_text: Text to locate the edit position (for longer documents)
            save_chunks_for_review: Save document sections (before/after) for usage review in dashboard
            use_large_model: Use larger AI model for more complex edits

        Returns:
            EditResponse with edit result

        Example:
            >>> edit = client.edit(
            ...     doc_id="doc123",
            ...     edit_uid=str(uuid4()),
            ...     edit_instructions="Change the title to 'My New Title'"
            ... )
        """
        request = EditRequest(
            edit_uid=edit_uid,
            edit_instructions=edit_instructions,
            lookup_text=lookup_text,
            save_chunks_for_review=save_chunks_for_review,
            use_large_model=use_large_model,
        )
        return self._edits_api.edit(doc_id, request)

    def read(self, doc_id: str, read_uid: Optional[str] = None) -> MarkdownReadResponse:
        """Read document as markdown.

        Args:
            doc_id: Document ID
            read_uid: Unique identifier for this read (auto-generated if not provided)

        Returns:
            MarkdownReadResponse with markdown content

        Example:
            >>> result = client.read(doc_id="doc123")
            >>> print(result.markdown)
        """
        if read_uid is None:
            read_uid = str(uuid4())

        request = MarkdownReadRequest(read_uid=read_uid)
        return self._markdown_api.read(doc_id, request)


class DocumentsNamespace:
    """Namespace for document operations."""

    def __init__(self, api: DocumentsAPI):
        """Initialize documents namespace.

        Args:
            api: DocumentsAPI instance
        """
        self._api = api

    def create(
        self,
        file: Union[str, Path, BinaryIO],
        ttl_seconds: Optional[int] = None,
        return_markdown: bool = False,
        tracked_changes: bool = False,
        author_name: str = "Anonymous",
    ) -> DocumentResponse:
        """Upload a document.

        Args:
            file: File path or file-like object
            ttl_seconds: Time to live in seconds (300-21600, i.e., 5 minutes to 6 hours)
            return_markdown: Whether to return markdown representation
            tracked_changes: Enable tracked changes mode - stores backup to show edits as Word track changes
            author_name: Author name for tracked changes attribution (only used if tracked_changes=True)

        Returns:
            DocumentResponse with document metadata

        Example:
            >>> doc = client.documents.create("document.docx", return_markdown=True)
            >>> print(f"Document ID: {doc.doc_id}")
        """
        return self._api.create(
            file, ttl_seconds, return_markdown, tracked_changes, author_name
        )

    def create_from_url(
        self,
        file_url: str,
        ttl_seconds: Optional[int] = None,
    ) -> DocumentResponse:
        """Create a document from a URL.

        Args:
            file_url: URL of the file to download and process
            ttl_seconds: Time to live in seconds (300-21600)

        Returns:
            DocumentResponse with document metadata

        Example:
            >>> doc = client.documents.create_from_url(
            ...     "https://example.com/document.docx"
            ... )
        """
        request = DocumentFromUrlRequest(file_url=file_url, ttl_seconds=ttl_seconds)
        return self._api.create_from_url(request)

    def list(self) -> ListDocumentsResponse:
        """List all documents.

        Returns:
            ListDocumentsResponse with list of documents

        Example:
            >>> docs = client.documents.list()
            >>> for doc in docs.documents:
            ...     print(f"{doc.name}: {doc.doc_id}")
        """
        return self._api.list()

    def download(self, doc_id: str, expires_in: int = 3600) -> DownloadDocumentResponse:
        """Get presigned URL to download a document.

        Args:
            doc_id: Document ID
            expires_in: URL expiration time in seconds (1-86400)

        Returns:
            DownloadDocumentResponse with download URL

        Example:
            >>> result = client.documents.download(doc_id="doc123")
            >>> print(f"Download URL: {result.download_url}")
        """
        return self._api.download(doc_id, expires_in)

    def exists(self, doc_id: str) -> DocumentExistsResponse:
        """Check if a document exists.

        Args:
            doc_id: Document ID

        Returns:
            DocumentExistsResponse with existence check result

        Example:
            >>> result = client.documents.exists(doc_id="doc123")
            >>> if result.exists:
            ...     print(f"Document exists: {result.document.name}")
        """
        return self._api.exists(doc_id)



"""Base API client."""

import json
from typing import Any, Dict, Optional
import requests
from ..exceptions import (
    AgentOfficeError,
    AuthenticationError,
    NotFoundError,
    ValidationError,
    RateLimitError,
    ServerError,
)


class BaseAPI:
    """Base class for all API clients."""
    
    def __init__(self, base_url: str, api_key: str, timeout: int = 60):
        """Initialize base API client.
        
        Args:
            base_url: Base URL for the API
            api_key: API key for authentication
            timeout: Request timeout in seconds
        """
        self.base_url = base_url.rstrip("/")
        self.api_key = api_key
        self.timeout = timeout
        self.session = requests.Session()
        self.session.headers.update({
            "Authorization": f"Bearer {api_key}",
        })
    
    def _handle_response(self, response: requests.Response) -> Any:
        """Handle API response and raise appropriate exceptions.
        
        Args:
            response: Response from API
            
        Returns:
            Parsed JSON response
            
        Raises:
            AgentOfficeError: For various error conditions
        """
        try:
            data = response.json()
        except json.JSONDecodeError:
            data = None
        
        if response.status_code == 200:
            return data
        elif response.status_code == 401:
            raise AuthenticationError(
                "Authentication failed. Check your API key.",
                status_code=401
            )
        elif response.status_code == 404:
            raise NotFoundError(
                data.get("detail", "Resource not found") if data else "Resource not found",
                status_code=404
            )
        elif response.status_code == 422:
            raise ValidationError(
                data.get("detail", "Validation error") if data else "Validation error",
                status_code=422
            )
        elif response.status_code == 429:
            raise RateLimitError(
                "Rate limit exceeded. Please try again later.",
                status_code=429
            )
        elif response.status_code >= 500:
            raise ServerError(
                data.get("detail", "Server error") if data else "Server error",
                status_code=response.status_code
            )
        else:
            raise AgentOfficeError(
                data.get("detail", f"Request failed with status {response.status_code}") if data 
                else f"Request failed with status {response.status_code}",
                status_code=response.status_code
            )
    
    def _get(self, path: str, params: Optional[Dict[str, Any]] = None) -> Any:
        """Make GET request.
        
        Args:
            path: API path
            params: Query parameters
            
        Returns:
            Parsed response
        """
        url = f"{self.base_url}{path}"
        response = self.session.get(url, params=params, timeout=self.timeout)
        return self._handle_response(response)
    
    def _post(
        self,
        path: str,
        json_data: Optional[Dict[str, Any]] = None,
        files: Optional[Dict[str, Any]] = None,
        data: Optional[Dict[str, Any]] = None,
    ) -> Any:
        """Make POST request.
        
        Args:
            path: API path
            json_data: JSON data to send
            files: Files to upload
            data: Form data to send
            
        Returns:
            Parsed response
        """
        url = f"{self.base_url}{path}"
        response = self.session.post(
            url,
            json=json_data,
            files=files,
            data=data,
            timeout=self.timeout
        )
        return self._handle_response(response)




"""Exception classes for Agent Office SDK."""


class AgentOfficeError(Exception):
    """Base exception for all Agent Office SDK errors."""
    
    def __init__(self, message: str, status_code: int = None):
        super().__init__(message)
        self.message = message
        self.status_code = status_code


class AuthenticationError(AgentOfficeError):
    """Raised when authentication fails (401)."""
    pass


class NotFoundError(AgentOfficeError):
    """Raised when a resource is not found (404)."""
    pass


class ValidationError(AgentOfficeError):
    """Raised when request validation fails (422)."""
    pass


class RateLimitError(AgentOfficeError):
    """Raised when rate limit is exceeded (429)."""
    pass


class ServerError(AgentOfficeError):
    """Raised when server returns a 5xx error."""
    pass




"""Document-related data models."""

from datetime import datetime
from typing import List, Optional
from dataclasses import dataclass


@dataclass
class ImageAnnotation:
    """Image annotation with caption."""
    
    image_id: str
    caption: str
    source: str  # 'generated' or 'existing'


@dataclass
class DocumentResponse:
    """Response from document creation."""
    
    doc_id: str
    name: str
    created_at: datetime
    file_type: Optional[str] = None
    image_annotations: Optional[List[ImageAnnotation]] = None
    markdown: Optional[str] = None
    
    @classmethod
    def from_dict(cls, data: dict) -> "DocumentResponse":
        """Create instance from API response dict."""
        image_annotations = None
        if data.get("imageAnnotations"):
            image_annotations = [
                ImageAnnotation(**ann) for ann in data["imageAnnotations"]
            ]
        
        return cls(
            doc_id=data["docId"],
            name=data["name"],
            created_at=datetime.fromisoformat(data["createdAt"].replace("Z", "+00:00")),
            file_type=data.get("fileType"),
            image_annotations=image_annotations,
            markdown=data.get("markdown"),
        )


@dataclass
class DocumentInfo:
    """Document metadata."""
    
    doc_id: str
    name: str
    size: int
    created_at: datetime
    updated_at: datetime
    content_type: str
    image_annotations: Optional[List[ImageAnnotation]] = None
    
    @classmethod
    def from_dict(cls, data: dict) -> "DocumentInfo":
        """Create instance from API response dict."""
        image_annotations = None
        if data.get("imageAnnotations"):
            image_annotations = [
                ImageAnnotation(**ann) for ann in data["imageAnnotations"]
            ]
        
        return cls(
            doc_id=data["docId"],
            name=data["name"],
            size=data["size"],
            created_at=datetime.fromisoformat(data["createdAt"].replace("Z", "+00:00")),
            updated_at=datetime.fromisoformat(data["updatedAt"].replace("Z", "+00:00")),
            content_type=data["contentType"],
            image_annotations=image_annotations,
        )


@dataclass
class DocumentExistsResponse:
    """Response for document exists check."""
    
    exists: bool
    document: Optional[DocumentInfo] = None
    
    @classmethod
    def from_dict(cls, data: dict) -> "DocumentExistsResponse":
        """Create instance from API response dict."""
        document = None
        if data.get("document"):
            document = DocumentInfo.from_dict(data["document"])
        
        return cls(
            exists=data["exists"],
            document=document,
        )


@dataclass
class ListDocumentsResponse:
    """Response for list documents."""
    
    documents: List[DocumentInfo]
    total_count: int
    
    @classmethod
    def from_dict(cls, data: dict) -> "ListDocumentsResponse":
        """Create instance from API response dict."""
        documents = [DocumentInfo.from_dict(doc) for doc in data["documents"]]
        return cls(
            documents=documents,
            total_count=data["totalCount"],
        )


@dataclass
class DownloadDocumentResponse:
    """Response for document download request."""
    
    doc_id: str
    download_url: str
    expires_in: int
    filename: str
    
    @classmethod
    def from_dict(cls, data: dict) -> "DownloadDocumentResponse":
        """Create instance from API response dict."""
        return cls(
            doc_id=data["docId"],
            download_url=data["downloadUrl"],
            expires_in=data["expiresIn"],
            filename=data["filename"],
        )


@dataclass
class DocumentFromUrlRequest:
    """Request to create document from URL."""
    
    file_url: str
    ttl_seconds: Optional[int] = None
    
    def to_dict(self) -> dict:
        """Convert to API request dict."""
        result = {"fileUrl": self.file_url}
        if self.ttl_seconds is not None:
            result["ttlSeconds"] = self.ttl_seconds
        return result




"""Documents API client."""

from typing import BinaryIO, Optional, Union
from pathlib import Path
import mimetypes
from .base import BaseAPI
from ..models import (
    DocumentResponse,
    DocumentExistsResponse,
    ListDocumentsResponse,
    DownloadDocumentResponse,
    DocumentFromUrlRequest,
)


class DocumentsAPI(BaseAPI):
    """Client for document operations."""

    def create(
        self,
        file: Union[str, Path, BinaryIO],
        ttl_seconds: Optional[int] = None,
        return_markdown: bool = False,
        tracked_changes: bool = False,
        author_name: str = "Anonymous",
    ) -> DocumentResponse:
        """Upload a document.

        Args:
            file: File path or file-like object
            ttl_seconds: Time to live in seconds (300-21600)
            return_markdown: Whether to return markdown representation
            tracked_changes: Enable tracked changes mode
            author_name: Author name for tracked changes attribution

        Returns:
            DocumentResponse with document metadata
        """
        # Handle file input
        if isinstance(file, (str, Path)):
            file_path = Path(file)
            with open(file_path, "rb") as f:
                file_content = f.read()
            filename = file_path.name
            # Guess content type from filename
            content_type, _ = mimetypes.guess_type(filename)
            if content_type is None:
                content_type = "application/octet-stream"
        else:
            file_content = file.read()
            filename = getattr(file, "name", "document")
            # Try to guess content type from filename if available
            content_type, _ = mimetypes.guess_type(filename)
            if content_type is None:
                content_type = "application/octet-stream"

        # Prepare form data with content type
        files = {"file": (filename, file_content, content_type)}
        data = {}
        if ttl_seconds is not None:
            data["ttl_seconds"] = ttl_seconds
        if return_markdown:
            data["return_markdown"] = return_markdown
        if tracked_changes:
            data["tracked_changes"] = tracked_changes
        if author_name:
            data["author_name"] = author_name

        response = self._post("/v1/documents/", files=files, data=data)
        return DocumentResponse.from_dict(response)

    def create_from_url(self, request: DocumentFromUrlRequest) -> DocumentResponse:
        """Create a document from a URL.

        Args:
            request: DocumentFromUrlRequest with file URL and options

        Returns:
            DocumentResponse with document metadata
        """
        response = self._post("/v1/documents/url", json_data=request.to_dict())
        return DocumentResponse.from_dict(response)

    def list(self) -> ListDocumentsResponse:
        """List all documents.

        Returns:
            ListDocumentsResponse with list of documents
        """
        response = self._get("/v1/documents/")
        return ListDocumentsResponse.from_dict(response)

    def download(self, doc_id: str, expires_in: int = 3600) -> DownloadDocumentResponse:
        """Get presigned URL to download a document.

        Args:
            doc_id: Document ID
            expires_in: URL expiration time in seconds (1-86400)

        Returns:
            DownloadDocumentResponse with download URL
        """
        params = {"expiresIn": expires_in}
        response = self._get(f"/v1/documents/{doc_id}", params=params)
        return DownloadDocumentResponse.from_dict(response)

    def exists(self, doc_id: str) -> DocumentExistsResponse:
        """Check if a document exists.

        Args:
            doc_id: Document ID

        Returns:
            DocumentExistsResponse with existence check result
        """
        response = self._get(f"/v1/documents/{doc_id}/exists")
        return DocumentExistsResponse.from_dict(response)



"""Markdown-related data models."""

from dataclasses import dataclass


@dataclass
class MarkdownReadRequest:
    """Request to read document as markdown."""
    
    read_uid: str
    
    def to_dict(self) -> dict:
        """Convert to API request dict."""
        return {"readUid": self.read_uid}


@dataclass
class MarkdownReadResponse:
    """Response from markdown read."""
    
    read_uid: str
    markdown: str
    time_to_generate: float
    document_name: str
    document_id: str
    
    @classmethod
    def from_dict(cls, data: dict) -> "MarkdownReadResponse":
        """Create instance from API response dict."""
        return cls(
            read_uid=data["readUid"],
            markdown=data["markdown"],
            time_to_generate=data["timeToGenerate"],
            document_name=data["documentName"],
            document_id=data["documentId"],
        )




"""Edits API client."""

from .base import BaseAPI
from ..models import EditRequest, EditResponse


class EditsAPI(BaseAPI):
    """Client for document edit operations."""
    
    def edit(self, doc_id: str, request: EditRequest) -> EditResponse:
        """Edit a document.
        
        Args:
            doc_id: Document ID
            request: EditRequest with edit instructions
            
        Returns:
            EditResponse with edit result
        """
        response = self._post(
            f"/v1/documents/{doc_id}/edits",
            json_data=request.to_dict()
        )
        return EditResponse.from_dict(response)




"""Edit-related data models."""

from typing import Optional
from dataclasses import dataclass


@dataclass
class EditRequest:
    """Request to edit a document."""

    edit_uid: str
    edit_instructions: str
    lookup_text: Optional[str] = None
    save_chunks_for_review: bool = False
    use_large_model: bool = False

    def to_dict(self) -> dict:
        """Convert to API request dict."""
        return {
            "editUid": self.edit_uid,
            "editInstructions": self.edit_instructions,
            "lookupText": self.lookup_text,
            "saveChunksForReview": self.save_chunks_for_review,
            "useLargeModel": self.use_large_model,
        }


@dataclass
class EditResponse:
    """Response from document edit."""

    edit_uid: str
    edit_applied: bool
    time_to_edit: float
    document_name: str
    document_id: str

    @classmethod
    def from_dict(cls, data: dict) -> "EditResponse":
        """Create instance from API response dict."""
        return cls(
            edit_uid=data["editUid"],
            edit_applied=data["editApplied"],
            time_to_edit=data["timeToEdit"],
            document_name=data["documentName"],
            document_id=data["documentId"],
        )



"""Agent Office Python SDK - AI-powered document editing for agentic workflows."""

from .client import AgentOffice
from .exceptions import (
    AgentOfficeError,
    AuthenticationError,
    NotFoundError,
    ValidationError,
    RateLimitError,
    ServerError,
)

__version__ = "0.0.1"
__all__ = [
    "AgentOffice",
    "AgentOfficeError",
    "AuthenticationError",
    "NotFoundError",
    "ValidationError",
    "RateLimitError",
    "ServerError",
]




"""Markdown API client."""

from .base import BaseAPI
from ..models import MarkdownReadRequest, MarkdownReadResponse


class MarkdownAPI(BaseAPI):
    """Client for markdown operations."""
    
    def read(self, doc_id: str, request: MarkdownReadRequest) -> MarkdownReadResponse:
        """Read document as markdown.
        
        Args:
            doc_id: Document ID
            request: MarkdownReadRequest with read UID
            
        Returns:
            MarkdownReadResponse with markdown content
        """
        response = self._post(
            f"/v1/documents/{doc_id}/markdown",
            json_data=request.to_dict()
        )
        return MarkdownReadResponse.from_dict(response)

